#!/usr/local/bin/perl

####################################################
#
# $Header: /cvsroot/disspam/disspam.pl,v 1.76 2005/01/05 20:34:11 mina Exp $
#
# DisSpam version 0.14
#
# Just one more script that tries to combat spam
#
# Written by Mina Naguib, http://www.topfx.com, webmaster@topfx.com
#
####################################################

use strict;
use Net::POP3;
use vars qw($VERSION $CONFFILENAME %RBL %MAILBOX $SENDMAIL $SMARTRELAY $DATAFILE $PIDFILE %HEADERHASHES $SPAMASSASSIN $SAOBJECT);

$VERSION = '0.14';

print "\nThis is DisSpam version $VERSION.  The time is now ", scalar localtime(time), "\n\n";

$CONFFILENAME = shift || usage();
(-f $CONFFILENAME) || die "Error: $CONFFILENAME is not a file\n";

parseconffile();

loadheaderhashes();

checkemail();

saveheaderhashes();

deletepidfile();

#
# This sub is the main loop. It loops through all the mailboxes defined in hash %MAILBOX
# , checks them for spam, and does whatever's appropriate if spam's found (backup to
# file / delete message / send bounceback etc...)
#
sub checkemail {
	my $host;
	my $username;
	my $password;
	my $email;
	my $pop;
	my $temp;
	my $messages;
	my $message_number;
	my $message;
	my $headers;
	my $reason;
	my $mbox;
	my $hash;
	local (*FH);

	foreach $mbox (sort keys %MAILBOX) {
		print "Processing [$mbox] ...\n";
		print "\n";
		$username = $MAILBOX{$mbox}{username};
		if (!$username) {
			warn "Error: Configuration [$mbox] is missing username. Skipping.\n";
			next;
		}
		$password = $MAILBOX{$mbox}{password};
		if (!$password) {
			warn "Error: Configuration [$mbox] is missing password. Skipping.\n";
			next;
		}
		$host = $MAILBOX{$mbox}{host};
		if (!$host) {
			warn "Error: Configuration [$mbox] is missing host. Skipping.\n";
			next;
		}
		$email = $MAILBOX{$mbox}{email};
		if (!$email) {
			warn "Error: Configuration [$mbox] is missing email. Skipping.\n";
			next;
		}
		$pop = Net::POP3->new($host, Timeout => 30);
		if (!$pop) {
			warn "Error: Could not connect to $host. Skipping.\n";
			next;
		}
		$temp = $pop->login($username, $password);
		if (!$temp) {
			warn "Error: Could not login to $host. Check username and password in configuration file. Skipping.\n";
			$pop->quit();
			next;
		}
		$messages = $pop->list();
		if (!$messages) {
			warn "Error: Could not retrieve message list\n";
			$pop->quit();
			next;
		}

		#
		# At this point we're connected to the POP server and have the list of messages
		# we loop over them, retrieve the headers/content if necessary, and process it
		#
		foreach $message_number (keys %{$messages}) {
			undef $headers;
			undef $message;
			if ($SPAMASSASSIN) {

				#
				# We need the entire message for spamassassin
				#
				print "\tRETRIEVING ENTIRE MESSAGE\n";
				$message = $pop->get($message_number);
				if (!$message) {
					print "\tERROR: FAILED TO RETRIEVE MESSAGE $message_number [$!] [$@]: SKIPPING\n";
					next;
				}
				$message = join("", @$message);
				($headers) = split(/\n\n/, $message);
			}
			else {

				#
				# We only need the headers for internal spamcheck
				#
				print "\tRETRIEVING MESSAGE HEADERS\n";
				$headers = $pop->top($message_number);
				if (ref($headers) ne "ARRAY") {
					next;
				}
				$headers = join("", @$headers);
			}

			#
			# Print from: and subject: lines:
			#
			print map { "\t$_\n" } ($headers =~ /^((?:from|subject):.+)$/gmi);
			if ($MAILBOX{$mbox}{checkonlyonce} && ($hash = gethash($headers)) && exists $HEADERHASHES{$hash}) {

				#
				# We've checked this message before
				# Strip out the "O" for old to denote that it's still new (in the mailbox)
				#
				$HEADERHASHES{$hash} =~ s/O//g;
				print "\tCHECKED BEFORE.  SKIPPING.\n";
				if ($HEADERHASHES{$hash} =~ /S/) {

					# It's old spam
					$reason = "Message previously detected as spam";
				}
				else {

					# It's old non-spam
					undef $reason;
				}
			}
			else {
				$reason = isspam($headers, \$message, $mbox);
				if ($MAILBOX{$mbox}{checkonlyonce}) {
					$hash ||= gethash($headers);
					$HEADERHASHES{$hash} = $reason ? "S" : "N";
				}
			}
			if ($reason) {
				print "\tTHAT IS SPAM ($reason)\n";
				if ($MAILBOX{$mbox}{deletespam}) {
					if ($MAILBOX{$mbox}{backupfile}) {
						if (!$message) {
							print "\tRETRIEVING ENTIRE MESSAGE\n";
							$message = $pop->get($message_number);
							if (!$message) {
								print "\tERROR: FAILED TO RETRIEVE MESSAGE $message_number [$!] [$@]: SKIPPING\n";
								next;
							}
							$message = join("", @$message);
						}
						print "\tSAVING MESSAGE TO BACKUP FILE\n";
						open(FH, ">>$MAILBOX{$mbox}{backupfile}") || die "Error: Could not open() $MAILBOX{$mbox}{backupfile} for appending: $!\n";
						print FH "From DisSpam ", scalar localtime(time), "\n";
						print FH $message, "\n";
						close(FH);
					}
					if ($MAILBOX{$mbox}{sendbounceback}) {
						sendbounceback($email, $headers, $reason);
					}
					$pop->delete($message_number);
					print "\tDELETED FROM SERVER\n";
				}
				else {
					print "\tNOT DELETED FROM SERVER\n";
				}
			}
			else {
				print "\tTHAT IS NOT SPAM\n";
				if ($MAILBOX{$mbox}{fetchcleanmessagesto}) {
					if ($MAILBOX{$mbox}{fetchcleanmessagesto} eq $MAILBOX{$mbox}{email}) {
						print "\tTHE FETCH-TO ADDRESS IS THE SAME AS THE MAILBOX ADDRESS\n";
						print "\tFETCH ABORTED TO AVOID LOOPBACK\n";
					}
					elsif (!$SENDMAIL && !$SMARTRELAY) {
						print "\tSENDMAIL OR SMARTRELAY NOT CONFIGURED. NOT FETCHING CLEAN MESSAGE\n";
					}
					elsif (!validemail($MAILBOX{$mbox}{fetchcleanmessagesto})) {
						print "\tTHE FETCH-TO ADDRESS [$MAILBOX{$mbox}{fetchcleanmessagesto}] IS INVALID. NOT FETCHING CLEAN MESSAGE\n";
					}
					else {
						if (!$message) {
							print "\tRETRIEVING ENTIRE MESSAGE\n";
							$message = $pop->get($message_number);
							if (!$message) {
								print "\tERROR: FAILED TO RETRIEVE MESSAGE $message_number [$!] [$@]: SKIPPING\n";
								next;
							}
							$message = join("", @$message);
						}
						print "\tRE-SENDING MESSAGE TO $MAILBOX{$mbox}{fetchcleanmessagesto}\n";
						send_mail($MAILBOX{$mbox}{email}, $MAILBOX{$mbox}{fetchcleanmessagesto}, $message) || die "Error: Failed to send: $@\n";
						print "\tDELETING MESSAGE FROM SERVER\n";
						$pop->delete($message_number);
					}
				}
			}
			print "\n";
		}
		$temp = $pop->quit();
		print "\n";
	}
}

#
# This sub takes an IP address and a mailbox hash key
# and returns true (1 or rbl name) if the address is banned in an RBL
# or returns 0 if it's not banned
#
sub isipbanned {
	my $ip   = shift || return "Empty mailserver IP";
	my $mbox = shift;
	my $dns  = new Net::DNS::Resolver;
	my $query;
	my $entry;
	my $rblquery;
	my $reversedip = join(".", reverse split(/\./, $ip));
	foreach (@{ $MAILBOX{$mbox}{exclude_ip} }) {

		if ($ip eq $_) {
			print "\tEXCLUDING $ip\n";
			return 0;
		}
	}
	foreach (keys %RBL) {
		$rblquery = "$reversedip.$_";
		if ($query = $dns->search($rblquery)) {
			foreach $entry ($query->answer) {
				if ($entry->type eq "A" && $entry->address =~ /^$RBL{$_}$/) {
					print "\tMATCHED $ip AT $_ (", $entry->address, ")\n";
					return $_;
				}
			}
		}
	}
	return 0;
}

sub usage {
	print <<"EOM";

Usage:

\t$0 file.conf

Where file.conf is the filename to read the configuration from.

See sample.conf for a sample of a configuration file, or readme.txt and configuration.txt for documentation on configuration file syntax.

EOM
	exit(0);
}

#
# This sub parses the config file, populating %RBL, %MAILBOX, $SENDMAIL, $SMARTRELAY, $DATAFILE, $PIDFILE
# Also does sanity checking and dies if it sees errors.
#
sub parseconffile {
	local (*FH);
	my $sectionname;
	my $key;
	my $value;
	my %global;

	%MAILBOX = ();

	open(FH, $CONFFILENAME) || die "Error: Could not open() $CONFFILENAME : $!\n";
	while (<FH>) {
		chomp;
		if (/^#/ || !$_) {
			next;
		}
		if (/^\s*\[\s*(.+?)\s*\]\s*$/) {
			if (exists $MAILBOX{ lc($1) }) {
				die "Error: Configuration file contains more than 1 section with name ", lc($1), "\n";
			}
			else {
				$sectionname = lc($1);
			}
		}
		elsif (/^\s*(.+?)\s*=\s*(.+?)\s*$/) {
			$key   = lc($1);
			$value = $2;
			if ($key =~ /^(checkonlyonce|sendbounceback)$/) {

				#
				# We want these values to default to false if unspecified:
				#
				$value = ($value =~ /^(y|1)/i) ? 1 : 0;
			}
			elsif ($key =~ /^(deletespam)$/) {

				#
				# We want these values to default to true if unspecified:
				#
				$value = ($value =~ /^(n|0)/i) ? 0 : 1;
			}
			elsif ($key =~ /^(exclude_from|exclude_ip|ban_from|ban_subject|ban_to)$/) {

				#
				# We want to convert these values to arrays (anonymous to be stored in hashkey)
				#
				$value = [ split(/\s*,\s*/, $value) ];
			}
			elsif ($key =~ /^(exclude_from_file|exclude_ip_file|ban_from_file|ban_subject_file|ban_to_file)$/) {

				#
				# We want to convert these values to arrays (anonymous to be stored in hashkey)
				# We also want to rename the key to exclude the _file part
				#
				$value = readfile($value);
				$key =~ s/_file$//;
			}
			$MAILBOX{$sectionname}{$key} = $value;
		}
	}
	close(FH);

	#
	# We check that they've defined an rbl and global section otherwise perl'll crap out when we access it
	# as a hash
	#
	if (!exists $MAILBOX{global}) {
		die "Error: Your configuration file does not contain a [GLOBAL] section\n";
	}
	if (!exists $MAILBOX{rbl} && $MAILBOX{global}{spamassassin} !~ /y|1/i) {
		die "Error: Your configuration file does not contain an [RBL] section and the global section does not enable SPAMASSASSIN\n";
	}

	#
	# We remove keys from %MAILBOX that are internal and not real mailbox definitions
	#
	if (ref($MAILBOX{rbl}) eq "HASH") {
		%RBL = %{ $MAILBOX{rbl} };
	}
	delete $MAILBOX{rbl};
	if (ref($MAILBOX{global}) eq "HASH") {
		%global = %{ $MAILBOX{global} };
	}
	delete $MAILBOX{global};

	#
	# We extract non-inheritable values from %global
	#
	$SENDMAIL = $global{sendmail};
	delete $global{sendmail};
	$SMARTRELAY = $global{smartrelay};
	delete $global{smartrelay};
	$DATAFILE = $global{datafile};
	delete $global{datafile};
	$PIDFILE = $global{pidfile};
	delete $global{pidfile};
	$SPAMASSASSIN = ($global{spamassassin} =~ /y|1/i) ? 1 : 0;
	delete $global{spamassassin};

	#
	# Now we inherit the settings from %global if they're not defined per mailbox
	#
	foreach $key (keys %global) {
		foreach (keys %MAILBOX) {
			$MAILBOX{$_}{$key} = (exists $MAILBOX{$_}{$key}) ? $MAILBOX{$_}{$key} : $global{$key};
		}
	}

	#
	# Now we do some sanity checking:
	#
	if (!$PIDFILE) {
		die "Error: You must specify a PIDFILE directive in the [GLOBAL] section in the configuration file\n";
	}
	elsif (-f $PIDFILE) {
		die "Error: Another instance of DisSpam is already running.  Please wait till it finishes.  If no DisSpam is currently running please delete PIDFILE [$PIDFILE]\n";
	}
	elsif (!scalar keys %RBL && !$SPAMASSASSIN) {
		die "Error: Configuration [RBL] section does not contain any RBL hostnames and global section does not enable SPAMASSASSIN.\n";
	}
	elsif (!scalar keys %MAILBOX) {
		die "Error: Configuration file does not define any mailboxes\n";
	}
	elsif ($SENDMAIL && (!-f $SENDMAIL || !-x $SENDMAIL)) {
		die "Error: Configured SENDMAIL [$SENDMAIL] does not exist or is not executable by me\n";
	}
	elsif ($SMARTRELAY && $SMARTRELAY !~ /^[a-z0-9._-]+$/i) {
		die "Error: Configured SMARTRELAY [$SMARTRELAY] does not appear to be a valid hostname\n";
	}
	elsif (!$DATAFILE) {
		die "Error: You must specify a DATAFILE directive in the [GLOBAL] section in the configuration file\n";
	}
	if (!open(FH, ">>$DATAFILE")) {
		die "Error: Could not open DATAFILE [$DATAFILE] for writing: $!\n";
	}
	else {
		close(FH);
	}
	if (!open(FH, ">$PIDFILE")) {
		die "Error: Could not open PIDFILE [$PIDFILE] for writing: $!\n";
	}
	else {
		print FH "$$\n";
		close(FH);
	}

	#
	# Load any required modules
	#
	if ($SMARTRELAY) {
		require Net::SMTP;
	}
	if ($SPAMASSASSIN) {
		require Mail::SpamAssassin;
	}
	else {
		require Net::DNS;
	}
}

#
# This sub takes in an email address, message headers, and a reason why the message was not delivered
# It then sends back an appropriate email (bounceback) saying the mail was not delivered and why
#
sub sendbounceback {
	my $email   = shift;
	my $headers = shift;
	my $reason  = shift;
	my $daemonfrom;
	my $to;
	my $replyto;
	my $message;

	($replyto) = $headers =~ /^Reply-to:\s*(.+)$/mi;
	($to)      = $headers =~ /^From:\s*(.+)$/mi;
	$to = $replyto || $to;
	if ($to =~ /([a-z0-9._-]+\@([a-z0-9-]+\.)+[a-z]{2,5})/i) {
		$to = $1;
	}
	else {
		return 0;
	}

	#
	# Prevent sending bounceback if certain conditions are met:
	#
	if ($headers =~ /^X-Loop/mi) {
		return 0;
	}
	elsif ($headers =~ /^Return-path:\s*<>/mi) {
		return 0;
	}
	elsif ($headers =~ /^Precedence:\s*(bulk|list|junk)/mi) {
		return 0;
	}

	$daemonfrom = $email;
	$daemonfrom =~ s/^(.+)\@(.+)$/MAILER-DAEMON\@$2/;

	print "\tSENDING BOUNCEBACK TO [$to]\n";
	$message = <<"EOM";
Return-path: <>
From: Mail Delivery Subsystem <>
To: $to
Subject: Returned mail: see transcript for details
X-Loop: $email
Auto-Submitted: auto-generated (failure)



This message was automatically generated by the
anti-SPAM software DisSpam.



A message you have sent to this address:

	* $email

Was not delivered due to the following reason:

	* $reason

More information about spam RBL blacklists in general:

	* http://www.ordb.org/faq/

This was diagnosed by the DisSpam utility monitoring
this email address.

To correct this problem, you may try to do the
following:

	* Study the reason mentioned above. Sometimes
	  it's self explanatory (you're banned, subject
	  of your message is banned etc...)
	* Re-send your message through another account
	  that does not use the above blacklisted
	  mailserver, sender name, subject...
	* Contact the owner of this mailbox via other
	  means and ask them to add you to their
	  exceptions list.
	* Contact the administrator of the above
	  blacklisted mailserver and ask them to
	  investigate this problem.  Including this
	  message would probably help them.


Please do not reply to this email.


Thank you.

DisSpam Anti-Spam Daemon
http://freshmeat.net/projects/disspam/


Complete headers from your message are included below this line.
=================================================================

$headers

EOM
	if (!send_mail($daemonfrom, $to, $message)) {
		print "\tFAILED TO SEND BOUNCEBACK: $@\n";
	}
}

#
# This sub takes in a scalar data and returns a unique hash corresponding to it
# (this is NOT fancy whatsoever. MD5 is probably much better, but I wanted less dependencies)
#
sub gethash {
	my $data = shift;
	my $hash;
	$hash = unpack("%16C*", $data);
	$hash .= "-" . length($data);
	$data =~ s/[^a-zA-Z0-9]//g;
	$data = join("", reverse sort split(//, $data));
	$data =~ s/((.)\2+)/length $1/ge;
	$hash .= "-" . $data;
	return $hash;
}

#
# This sub loads the datafile into the hash %HEADERHASHES
#
sub loadheaderhashes {
	local (*FH);
	my ($key, $value);
	open(FH, $DATAFILE) || die "Error: Could not open DATAFILE [$DATAFILE] for input: $!\n";
	%HEADERHASHES = ();
	while (<FH>) {
		chomp;
		($key, $value) = split(/\s*=\s*/);
		$key || next;
		$value ||= "O";
		$HEADERHASHES{$key} = $value;
	}
	close(FH);
}

#
# This sub saves the hash %HEADERHASHES into the datafile to be read the next run
#
# The values for the keys are either "O" (old unknown), "OS" (old spam), "ON" (old not spam), "S" (spam) or "N" (not spam)
# It only saves keys with a value of "S" or "N"
# Keys with a value of "O*" mean we read them from the datafile originally, but they no
# longer exist in the mailboxes, so no need to keep them laying around
#
sub saveheaderhashes {
	local (*FH);
	open(FH, ">$DATAFILE") || die "Error: Could not open DATAFILE [$DATAFILE] for output: $!\n";
	foreach (keys %HEADERHASHES) {
		$_ || next;
		if ($HEADERHASHES{$_} !~ /^O/) {
			print FH "$_=$HEADERHASHES{$_}\n";
		}
	}
	close(FH);
}

#
# This sub deletes the PIDFILE we created in the beginning
#
sub deletepidfile {
	if (!-f $PIDFILE) {
		die "Error: Could not delete PIDFILE [$PIDFILE] because it does not exist\n";
	}
	elsif (!unlink($PIDFILE)) {
		die "Error: Could not delete PIDFILE [$PIDFILE]: $!\n";
	}
}

#
# This sub takes an email address and returns if it's valid or not
#
sub validemail {
	my $email = shift;
	if ($email =~ /^[a-z0-9._-]+\@([a-z0-9-]+\.)+[a-z]{2,4}$/i) {
		return 1;
	}
	else {
		return 0;
	}
}

#
# This sub sends an email
# Takes from email, to email, message content
# Returns true on successful send, false on failure
# If failed, sets $@ to an appropriate error message;
#
# Note: Supplied message MUST contain To: header, an empty, line, then message content
#
sub send_mail {
	my ($from, $to, $message) = @_;
	my $smtp;
	local (*SMH);

	#
	# Some sanity:
	#
	validemail($from) or ($@ = "From address not valid" and return undef);
	validemail($to)   or ($@ = "To address not valid"   and return undef);
	$message          or ($@ = "Empty message supplied" and return undef);

	#
	# Prefer smartrelay first:
	#
	if ($SMARTRELAY) {
		$smtp = new Net::SMTP($SMARTRELAY) or ($@ = "Failed to create a new Net::SMTP object with smartrelay [$SMARTRELAY]: $! $@" and return undef);
		$smtp->mail($from)        or ($@ = "Failed to set SMTP sender: $! $@"     and return undef);
		$smtp->to($to)            or ($@ = "Failed to set SMTP recipient: $! $@"  and return undef);
		$smtp->data()             or ($@ = "Failed to initiate SMTP data: $! $@"  and return undef);
		$smtp->datasend($message) or ($@ = "Failed to send SMTP data: $! $@"      and return undef);
		$smtp->dataend()          or ($@ = "Failed to terminate SMTP data: $! $@" and return undef);
		$smtp->quit();
		return 1;
	}

	#
	# Otherwise try sendmail
	#
	elsif ($SENDMAIL) {
		open(SMH, "| $SENDMAIL -tiN never") or ($@ = "Failed to launch SENDMAIL [$SENDMAIL]: $!" and return undef);

		#
		# Make sure "to" header in message is $to
		if ($message =~ s/^To:.*$/To: $to/mi) {

			# Message already had headers, we replaced it
		}
		else {

			# Message did not have headers. we add
			$message = "To: $to\n\n$message";
		}
		print SMH $message;
		close(SMH);
		return 1;
	}

	#
	# If neither exist, we can't send an email:
	#
	else {
		$@ = "No SENDMAIL or SMARTRELAY directives defined in [global] section in configuration file";
		return undef;
	}
}

#
# This sub is passed headers, (optional) message reference, and mbox
# Should return true (an error message) if it received spam, false if it's not spam
#
sub isspam {
	my $headers    = shift || die "Error: Headers not supplied to isspam\n";
	my $messageref = shift;
	my $mbox       = shift || die "Error: mbox not supplied to isspam\n";
	my @header;
	my ($from, $subject, $to);
	my %ip;
	my $temp;
	my ($status, $score);

	#
	# Sanity:
	#
	if ($messageref && ref($messageref) ne "SCALAR") {
		die "Error: Supplied messageref to isaspam is not a reference to a scalar: $messageref\n";
	}

	#
	# First, let's split the headers into an array
	#
	foreach (split(/\n+/, $headers)) {
		if (/^\s/) {
			$header[$#header] .= $_;
		}
		else {
			push(@header, $_);
		}
	}

	#
	# Now let's analyze the headers for mailbox-specific from/subject bans
	# (No spam detection yet)
	#
	foreach (@header) {
		if (/^Received:/ && /\[(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\]/) {
			$ip{$1} = 1;
		}
		elsif (/^From:\s*(.+?)\s*$/) {
			$from = $1;
			foreach (@{ $MAILBOX{$mbox}{exclude_from} }) {
				if ($from =~ /$_/i) {
					print "\tEXCLUDING $_\n";
					return 0;
				}
			}
			foreach (@{ $MAILBOX{$mbox}{ban_from} }) {
				if ($from =~ /$_/i) {
					print "\tBAN FROM MATCHED $_\n";
					return "Sender is banned from emailing this recipient";
				}
			}
		}
		elsif (/^To:\s*(.+?)\s*$/) {
			$to = $1;
			foreach (@{ $MAILBOX{$mbox}{ban_to} }) {
				if ($to =~ /$_/i) {
					print "\tBAN TO MATCHED $_\n";
					return "Recipient is banned from receiving any mail";
				}
			}
		}
		elsif (/^Subject:\s*(.+?)\s*$/) {
			$subject = $1;
			foreach (@{ $MAILBOX{$mbox}{ban_subject} }) {
				if ($subject =~ /$_/i) {
					print "\tBAN SUBJECT MATCHED $_\n";
					return "Message genre/subject is banned from being received by this recipient";
				}
			}
		}
	}

	#
	# If we've reached here, then it's time to truly decide if the message is spam or not
	#
	if ($SPAMASSASSIN) {

		#
		# Use spam assassin for detection
		#
		$SAOBJECT ||= Mail::SpamAssassin->new();    #only recreate SA object if necessary
		$status = $SAOBJECT->check_message_text($$messageref);
		$score  = $status->get_hits() . "/" . $status->get_required_hits();
		print "\tSPAMASSASSIN SCORE: $score\n";
		if ($status->is_spam()) {
			$status->finish();
			return "Message detected as spam by spamassassin ($score)";
		}
		else {

			#give email SpamAssassin headers
			$status->rewrite_mail();
			eval { $$messageref = $status->get_message(); };
			if ($@) {

				#
				# Older versions of spamassassin don't have that function
				#
				$$messageref = $status->get_full_message_text();
			}
			$status->finish();
		}
	}
	else {

		#
		# Use built-in detection via RBLs
		#
		foreach (keys %ip) {
			if ($temp = isipbanned($_, $mbox)) {
				return "An internet e-mail server this message passed through [$_] is banned by RBL blacklist [$temp]";
			}
		}
	}

	#
	# Message is not spam if it reached here
	#
	return undef;

}

#
# This sub gets passed a filename
# and it returns an array (or arrayref) containint each (chomped) line in the file
# dies if file can't be opened
#
sub readfile {
	my $filename = shift || die "Error: Filaname not supplied to readfile";
	local (*FH);
	my @data;
	open(FH, $filename) || die "Error: Failed to open $filename: $!\n";
	while (<FH>) {
		chomp;
		push(@data, $_);
	}
	close(FH);
	return wantarray ? @data : \@data;
}
